import 'package:_private_core/_private_core.dart';
import 'package:_private_core_network/network_resources/auth/models/auth_industry.dart';
import 'package:_private_core_network/network_resources/resources.dart';
import 'package:_private_core_network/utils.dart';
import 'package:dio/dio.dart';

class _MyAppEndpoint {
  _MyAppEndpoint._();
  static String resetPasswordByEmail() => "resetPasswordByEmail";
  static String confirmResetProPasswordCode() => "confirmResetProPasswordCode";
  static String updateNewProPasswordByToken() => "updateNewProPasswordByToken";
  static String confirmResetUserPasswordCode() =>
      "confirmResetUserPasswordCode";
  static String updateNewUserPasswordByToken() =>
      "updateNewUserPasswordByToken";
  static String setThumbnailForUser() => "setThumbnailForUser";
  static String setThumbnailForPro() => "setThumbnailForPro";

  static String getServiceCatForIndustry() =>
      "v2/common/getServiceCatForIndustry";
}

abstract class MyAppApi {
  Future<NetworkResponse> resetPasswordByEmail(params);
  Future<NetworkResponse> confirmResetProPasswordCode(params);
  Future<NetworkResponse> updateNewProPasswordByToken(params);
  Future<NetworkResponse> confirmResetUserPasswordCode(params);
  Future<NetworkResponse> updateNewUserPasswordByToken(params);

  Future<NetworkResponse> setThumbnailForUser(bytes, filePath,
      [ProgressCallback? callback]);
  Future<NetworkResponse> setThumbnailForPro(bytes, filePath,
      [ProgressCallback? callback]);

  Future<NetworkResponse> getUserInfo(params);
  Future<NetworkResponse> getProInfo(params);

  Future<NetworkResponse> getServiceCatForIndustry(params);
}

class MyAppApiImp extends MyAppApi {
  @override
  Future<NetworkResponse> resetPasswordByEmail(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(requiredToken: false)
            .post(_MyAppEndpoint.resetPasswordByEmail(), data: (params));
        return NetworkResponse.fromResponse(response,
            converter: (json) => json, prefix: "");
      },
    );
  }

  @override
  Future<NetworkResponse> confirmResetProPasswordCode(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(requiredToken: false)
            .post(_MyAppEndpoint.confirmResetProPasswordCode(), data: (params));
        return NetworkResponse.fromResponse(response,
            converter: (json) => ConfirmOtp.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> confirmResetUserPasswordCode(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(requiredToken: false).post(
            _MyAppEndpoint.confirmResetUserPasswordCode(),
            data: (params));
        return NetworkResponse.fromResponse(response,
            converter: (json) => ConfirmOtp.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> updateNewProPasswordByToken(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(requiredToken: false)
            .post(_MyAppEndpoint.updateNewProPasswordByToken(), data: (params));
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> updateNewUserPasswordByToken(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(requiredToken: false).post(
            _MyAppEndpoint.updateNewUserPasswordByToken(),
            data: (params));
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> setThumbnailForUser(bytes, filePath,
      [ProgressCallback? callback]) async {
    return await handleNetworkError(
      proccess: () async {
        String fileName = filePath.split('/').last;
        FormData formData = FormData.fromMap({
          "fileName": MultipartFile.fromBytes(bytes, filename: fileName),
        });
        final token = await appPrefs.getNormalToken();
        Response response = await AppClient(requiredToken: true, token: token)
            .post(_MyAppEndpoint.setThumbnailForUser(),
                data: formData, onSendProgress: callback);
        return NetworkResponse.fromResponse(response,
            converter: (json) => json['fileName']);
      },
    );
  }

  @override
  Future<NetworkResponse> setThumbnailForPro(bytes, filePath,
      [ProgressCallback? callback]) async {
    return await handleNetworkError(
      proccess: () async {
        String fileName = filePath.split('/').last;
        FormData formData = FormData.fromMap({
          "fileName": MultipartFile.fromBytes(bytes, filename: fileName),
        });
        final token = await appPrefs.getNormalToken();
        Response response = await AppClient(requiredToken: true, token: token)
            .post(_MyAppEndpoint.setThumbnailForPro(),
                data: formData, onSendProgress: callback);
        return NetworkResponse.fromResponse(response,
            converter: (json) => json['fileName']);
      },
    );
  }

  @override
  Future<NetworkResponse> getUserInfo(params) async {
    return await handleNetworkError(
      proccess: () async {
        final token = await appPrefs.getNormalToken();
        Response response = await AppClient(requiredToken: true, token: token)
            .get(AppEndpoint.getUserInfo(), queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => UserModel.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> getProInfo(params) async {
    return await handleNetworkError(
      proccess: () async {
        final token = await appPrefs.getNormalToken();
        Response response = await AppClient(requiredToken: true, token: token)
            .get(AppEndpoint.getProInfo(), queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => ProInfo.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> getServiceCatForIndustry(params) async {
    return await handleNetworkError(
      proccess: () async {
        final token = await appPrefs.getNormalToken();
        Response response = await AppClient(requiredToken: true, token: token)
            .get(_MyAppEndpoint.getServiceCatForIndustry(),
                queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) =>
                (json as List).map((e) => AuthIndustry.fromJson(e)).toList());
      },
    );
  }
}
